;   ͻ
;     FILE NAME: PWM.ASM                SCOPE:                           
;   Ĵ                                   
;     LANGUAGE: PIC 16C56 MPALC          PWM Motor control with PIC56    
;   Ĵ                                   
;     VERSION: 1    REVISION: 1                                         
;   Ķ
;     DATE:  10/05/94                   AUTHOR: Sam Lowry                
;   ͼ

LIST P=16C56

         Same      EQU       .1

         PC        EQU       0x002
         SWR       EQU       0x003


         Carry     EQU       .0
         ZeroB     EQU       .2

         OptionSet EQU       B'00111100'
         PortAStat EQU       B'00001000'
         PortASet  EQU       B'00000110'
         PortBStat EQU       B'11111111'
         PortBSet  EQU       B'00000000'

         DEF_FREQ  EQU  .80                 ; Must be even!
         DEF_DUTY  EQU  DEF_FREQ/2
         DEF_ACCEL EQU  .1
         DEF_RES   EQU  .2                  ; Must be 50


         Port_A    EQU  0x005
         Port_B    EQU  0x006
         Freq      EQU  0x007
         Duty      EQU  0x008
         Accel     EQU  0x009
         Count     EQU  0x00A
         Temp      EQU  0x00B
         Temp1     EQU  0x00C
         Resol     EQU  0x00D

         GO_IN     EQU  0
         DIR_IN    EQU  1

         PWM_OUT   EQU  0
         ENABLE    EQU  1

; ==================================
         ORG       0x000

DELAY    MOVWF     Count
DELAY1   MOVLW     .20                      ; 400 nS
         MOVWF     Temp                     ;
DELAY2   MOVLW     .82                      ; 400 nS
         MOVWF     Temp1                    ;
DELAY3   DECFSZ    Temp1,Same               ; 600 nS x Temp1
         GOTO      DELAY3                   ;
         DECFSZ    Temp,Same                ; 600 nS
         GOTO      DELAY2                   ;
         DECFSZ    Count,Same               ; 600 nS
         GOTO      DELAY1                   ;
         RETLW     0

; ==================================
;          Here is MAIN

         ORG       0x0FF

Main     MOVLW     PortASet
         MOVWF     Port_A
         MOVLW     PortBSet
         MOVWF     Port_B
         MOVLW     PortAStat
         TRIS      Port_A
         MOVLW     PortBStat
         TRIS      Port_B

         MOVLW     OptionSet
         OPTION
         CLRF      0x001

         MOVLW     DEF_FREQ
         MOVWF     Freq
         MOVLW     DEF_DUTY
         MOVWF     Duty

         MOVLW     .100
         CALL      DELAY

         BCF       Port_A,ENABLE            ; Enable bridge


MainLoop MOVFW     Duty                     ; 1 cyc Ŀ
         MOVWF     Count                    ; 1 cyc   OFF
         BSF       Port_A,PWM_OUT           ; 1 cyc 

MHiLoop  DECFSZ    Count,Same               ; 1 cyc Ŀ
         GOTO      MHiLoop                  ; 2 cyc  
                                            ;        
         MOVFW     Freq                     ; 1      
         MOVWF     Count                    ; 1       ON 5 + Loop
         MOVFW     Duty                     ; 1      
         SUBWF     Count,Same               ; 1      
         BCF       Port_A,PWM_OUT           ; 1 cyc 

MLLoop   DECFSZ    Count,Same
         GOTO      MLLoop
                                            ;            TEST INPUT
         BTFSC     Port_B,GO_IN             ; 1 cyc      0 = START
         GOTO      MainLoop                 ; 2          1 = STOP

; ---------------------------------------------------------------------

Move     BTFSS     Port_B,DIR_IN            ; Direction  1 = UP
         GOTO      MOVE_DW                  ;            0 = DOWN
         GOTO      MOVE_UP


; *****************************************************************
; *****************************************************************

; ------------ ACCEL TO THE LEFT -------------

MOVE_DW  NOP
         MOVLW     DEF_ACCEL
         MOVWF     Accel
         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

AC1_LOOP BSF       Port_A,PWM_OUT

         DECFSZ    Resol,Same               ; *
         GOTO      NoChg11                  ; *

         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

         MOVFW     Accel
         SUBWF     Duty,0                   ; Duty = Duty - Accel
         BTFSS     SWR,Carry                ; IF Duty < Accel THEN End
         GOTO      Ac1_End

         BTFSC     SWR,ZeroB                ; IF Duty = Accel THEN End
         GOTO      Ac1_End

         GOTO      GoOn11                   ; *

NoChg11  NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         MOVFW     Duty                     ; *

GoOn11   MOVWF     Duty                     ; *
         MOVWF     Count                    ; Count = Duty - Accel

Acc1_L1  DECFSZ    Count,Same
         GOTO      Acc1_L1

         BCF       Port_A,PWM_OUT

         MOVFW     Duty                     ; Count = Freq - Duty
         SUBWF     Freq,0
         MOVWF     Count
         NOP
         NOP
         NOP
         NOP

Acc1_L2  DECFSZ    Count,Same
         GOTO      Acc1_L2

         GOTO      AC1_LOOP

; ------------ MAX SPEED ON LEFT -------------

Ac1_End  MOVLW     .1
         MOVWF     Duty

MAX_LEFT MOVFW     Duty                     ; 1 cyc Ŀ
         MOVWF     Count                    ; 1 cyc   OFF
         BSF       Port_A,PWM_OUT           ; 1 cyc 

LHiLoop  DECFSZ    Count,Same               ; 1 cyc Ŀ
         GOTO      LHiLoop                  ; 2 cyc  
                                            ;        
         MOVFW     Freq                     ; 1      
         MOVWF     Count                    ; 1       ON 5 + Loop
         MOVFW     Duty                     ; 1      
         SUBWF     Count,Same               ; 1      
         NOP                                ; 1      
         BCF       Port_A,PWM_OUT           ; 1 cyc 

LLLoop   DECFSZ    Count,Same
         GOTO      LLLoop
                                            ;            TEST INPUT
         BTFSS     Port_B,GO_IN             ; 1 cyc
         GOTO      MAX_LEFT                 ; 2


; ------------ BRAKE FROM THE LEFT -----------

BRK_LEFT MOVLW     DEF_ACCEL
         MOVWF     Accel
         NOP
         NOP
         NOP
         NOP
         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

BR1_LOOP BSF       Port_A,PWM_OUT

         DECFSZ    Resol,Same               ; *
         GOTO      NoChg12                  ; *

         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

         MOVFW     Accel
         ADDWF     Duty,Same                ; Duty = Duty + Accel
         MOVLW     DEF_DUTY
         SUBWF     Duty,0
         BTFSC     SWR,Carry                ; IF Duty > 50 THEN End
         GOTO      Br1_End

         BTFSC     SWR,ZeroB                ; IF Duty = Accel THEN End
         GOTO      Br1_End

         GOTO      GoOn12                   ; *

NoChg12  NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         MOVFW     Duty                     ; *

GoOn12   MOVFW     Duty
         MOVWF     Count                    ; Count = Duty + Accel

Brk1_L1  DECFSZ    Count,Same
         GOTO      Brk1_L1

         BCF       Port_A,PWM_OUT

         MOVFW     Duty                     ; Count = Freq - Duty
         SUBWF     Freq,0
         MOVWF     Count
         NOP
         NOP
         NOP
         NOP

Brk1_L2  DECFSZ    Count,Same
         GOTO      Brk1_L2

         GOTO      BR1_LOOP

Br1_End  MOVLW     DEF_DUTY
         MOVWF     Duty
         GOTO      MainLoop


; *****************************************************************
; *****************************************************************

; ------------ ACCEL TO THE RIGHT ------------

MOVE_UP  MOVLW     DEF_ACCEL
         MOVWF     Accel
         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

AC2_LOOP BSF       Port_A,PWM_OUT

         DECFSZ    Resol,Same               ; *
         GOTO      NoChg21                  ; *

         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

         MOVFW     Accel
         ADDWF     Duty,Same                ; Duty = Duty + Accel
         MOVFW     Freq
         SUBWF     Duty,0
         BTFSC     SWR,Carry                ; IF Duty > Freq THEN End
         GOTO      Ac2_End

         BTFSC     SWR,ZeroB                ; IF Duty = Freq THEN End
         GOTO      Ac2_End

         GOTO      GoOn21                   ; *

NoChg21  NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         MOVFW     Duty                     ; *

GoOn21   MOVFW     Duty
         MOVWF     Count                    ; Count = Duty - Accel

Acc2_L1  DECFSZ    Count,Same
         GOTO      Acc2_L1

         BCF       Port_A,PWM_OUT

         MOVFW     Duty                     ; Count = Freq - Duty
         SUBWF     Freq,0
         MOVWF     Count
         NOP
         NOP
         NOP
         NOP

Acc2_L2  DECFSZ    Count,Same
         GOTO      Acc2_L2

         GOTO      AC2_LOOP

; ------------ MAX SPEED ON RIGHT ------------

Ac2_End  MOVFW     Freq
         MOVWF     Duty
         DECF      Duty

MAX_RIGH MOVFW     Duty                     ; 1 cyc Ŀ
         MOVWF     Count                    ; 1 cyc   OFF
         BSF       Port_A,PWM_OUT           ; 1 cyc 

RHiLoop  DECFSZ    Count,Same               ; 1 cyc Ŀ
         GOTO      RHiLoop                  ; 2 cyc  
                                            ;        
         MOVFW     Freq                     ; 1      
         MOVWF     Count                    ; 1       ON 5 + Loop
         MOVFW     Duty                     ; 1      
         SUBWF     Count,Same               ; 1      
         NOP                                ; 1      
         BCF       Port_A,PWM_OUT           ; 1 cyc 

RLLoop   DECFSZ    Count,Same
         GOTO      RLLoop
                                            ;            TEST INPUT
         BTFSS     Port_B,GO_IN             ; 1 cyc
         GOTO      MAX_RIGH                 ; 2


; ------------ BRAKE FROM THE RIGHT ----------

BRK_RIGH MOVLW     DEF_ACCEL
         MOVWF     Accel
         NOP
         NOP
         NOP
         NOP
         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

BR2_LOOP BSF       Port_A,PWM_OUT

         DECFSZ    Resol,Same               ; *
         GOTO      NoChg22                  ; *

         MOVLW     DEF_RES                  ; *
         MOVWF     Resol                    ; *

         MOVFW     Accel
         SUBWF     Duty,Same                ; Duty = Duty - Accel
         MOVLW     DEF_DUTY
         SUBWF     Duty,0
         BTFSS     SWR,Carry                ; IF Duty < 50 THEN End
         GOTO      Br2_End

         BTFSC     SWR,ZeroB                ; IF Duty = DefDuty THEN End
         GOTO      Br2_End

         GOTO      GoOn22                   ; *

NoChg22  NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         NOP                                ; *
         MOVFW     Duty                     ; *

GoOn22   MOVFW     Duty
         MOVWF     Count

Brk2_L1  DECFSZ    Count,Same
         GOTO      Brk2_L1

         BCF       Port_A,PWM_OUT

         MOVFW     Duty                     ; Count = Freq - Duty
         SUBWF     Freq,0
         MOVWF     Count
         NOP
         NOP
         NOP
         NOP

Brk2_L2  DECFSZ    Count,Same
         GOTO      Brk2_L2

         GOTO      BR2_LOOP

Br2_End  MOVLW     DEF_DUTY
         MOVWF     Duty
         GOTO      MainLoop


; ----------------------------
         ORG       0x3FF
         GOTO      Main
         END
